"""
python -m unittest palmotif/tests/test_compute.py
"""
import unittest
import numpy as np
import pandas as pd
from os.path import join as opj

from palmotif import *

from palmotif.tests.sequences import seqs1, seqs2


class TestCompute(unittest.TestCase):

    def test_compute_motif(self):
        motif = compute_motif(seqs1, reference_freqs=None, weights=None, align_first=False, gap_reduce=None, alphabet=None)
        self.assertTrue(True)

    def test_reference_freqs_motif(self):
        motif = compute_motif(seqs1, reference_freqs=uniprot_frequency, weights=None, align_first=False, gap_reduce=None, alphabet=None)
        self.assertTrue(True)

    def test_compute_motif(self):
        motif = compute_relative_motif(seqs1, seqs2)
        self.assertTrue(True)

    def test_pal_no_ref(self):
        mixed_seqs = seqs1 + [s[:9] for s in seqs1]
        motif, out = compute_pal_motif(mixed_seqs[0], mixed_seqs, refs=None, gopen=3, gextend=3, matrix=None, ref_freqs=None)

    def test_pal_no_ref2(self):
        mixed_seqs1 = seqs1 + [s[:9] for s in seqs1]
        mixed_seqs2 = seqs2 + [s[:9] for s in seqs2]
        motif, out = compute_pal_motif(mixed_seqs1[1], mixed_seqs1, refs=None, gopen=3, gextend=3, matrix=None, ref_freqs=uniprot_frequency)

    def test_pal_yes_ref(self):
        mixed_seqs1 = seqs1 + [s[:9] for s in seqs1]
        mixed_seqs2 = seqs2 + [s[:9] for s in seqs2]
        motif, out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None)

    def test_pal_loglik(self):
        mixed_seqs1 = seqs1 + [s[:9] for s in seqs1]
        mixed_seqs2 = seqs2 + [s[:9] for s in seqs2]
        motif, out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None, bootstrap_samples=100)

    def test_pal_yes_ref_with_liklihood(self):
        mixed_seqs1 = ["A","A","A","A"]
        mixed_seqs2 = ["T","T","T","T"]
        motif, out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None, bootstrap_samples=100)
        assert out['loglik'].iloc[0] < 0.000000000000000000000000001 

    def test_pal_yes_ref_with_liklihood2(self):
        """Assert that number is very small in case where seqs match ref when smooth = True, and 0 if smooth = false """
        mixed_seqs1 = ["A","A","A"]
        mixed_seqs2 = ["A","A","A"]
        _, out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None, pseudo_count=1e-6)
        assert out['loglik'].iloc[0] < 0.0001#,#[np.log(1**3)])#atol=1)
        _, out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None, pseudo_count=0)
        assert out['loglik'].iloc[0] == np.log(1**3)

    def test_pal_yes_ref_with_liklihood3(self):
        """simple tests that log liklihood is expected in simple bernoulli trial i.e. (.5)^4 if smooth = False and very close if smooth is True"""
        mixed_seqs1 = ["A","A","A","A"]
        mixed_seqs2 = ["T","T","A","A"]
        _,out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None, pseudo_count=1e-6)
        assert np.isclose(out['loglik'].iloc[0], np.log(.5**4), rtol=0.00001)
        _,out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None, pseudo_count=0)
        assert out['loglik'].iloc[0] == np.log(.5**4)

    def test_pal_yes_ref_with_liklihood4(self):
        """
        This is an explicit test that laplace smoothing is working such that the log liklihood is not -inf
        even thought T never appears in the refernce set.
        """
        mixed_seqs1 = ["T","A","A"]
        mixed_seqs2 = ["A","A","A"]
        _,out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None)
        assert out['loglik'].iloc[0] > np.NINF
  
    def test_pal_yes_ref_with_liklihood_no_smoothing(self):
        """
        With no smoothing, get -inf when amino in seqs not in ref seqs
        """
        mixed_seqs1 = ["T","A","A"]
        mixed_seqs2 = ["A","A","A"]
        _,out = compute_pal_motif(mixed_seqs1[2], mixed_seqs1, refs=mixed_seqs2, gopen=3, gextend=3, matrix=None, pseudo_count=0)
        assert out['loglik'].iloc[0] == np.NINF

if __name__ == '__main__':
    unittest.main()



from .mpl_plot import mpl_logo
from .svg_logo import svg_logo
from .compute_logo import *

__all__ = ['compute_motif',
           'mpl_logo',
           'compute_pal_motif',
           'uniprot_frequency',
           'compute_relative_motif',
           'svg_logo']



import matplotlib.pyplot as plt
import matplotlib.patheffects
import matplotlib.transforms as mtrans
from matplotlib import transforms
import numpy as np
from . import aacolors

__all__ = ['mpl_logo']

class Scale(matplotlib.patheffects.RendererBase):
    def __init__(self, sx=1., sy=1.):
        self._sx = sx
        self._sy = sy

    def draw_path(self, renderer, gc, tpath, affine, rgbFace):
        affine = affine.identity().scale(self._sx, self._sy) + affine
        renderer.draw_path(gc, tpath, affine, rgbFace)

def _extend_bbox(bbox, ext):
    if ext.x0 < bbox[0, 0]:
        bbox[0, 0] = ext.x0
    if ext.y0 < bbox[0, 1]:
        bbox[0, 1] = ext.y0
    if ext.x1 > bbox[1, 0]:
        bbox[1, 0] = ext.x1
    if ext.y1 < bbox[1, 1]:
        bbox[1, 1] = ext.y1
    return bbox

def mpl_logo(x, y, motif, axh=None, fontsize=16, color_scheme='shapely'):
    """Sequence logo of the motif at data coordinates x,y using matplotlib.
    
    Not a great solution since logos are difficult to place in coordinate space and
    may be distorted by zooming after it is plotted.

    Inspiration:
    https://github.com/saketkc/motif-logos-matplotlib/blob/master/Motif%20Logos%20using%20matplotlib.ipynb

    Parameters
    ----------
    x,y : float
        Position for the bottom-left corner of the logo in the axes data coordinates
    motif : pd.DataFrame
        Matrix of scores for each AA symbol (index, rows) and each
        column in an alignment (columns). Values will be used to scale
        each symbol linearly.
    axh : matplotlib axes handle
        Will use plt.gca() if None
    fontsize : float
        Pointsize of font passed to axh.text
    color_scheme : str
        An aa_grouping instance from aacolors.py or custom define.
        Options: nucleotide, base_pairing, hydrophobicity, chemistry, charge, taylor, logojs, shapely

    Returns
    -------
    letters : list of lists of tuples (Text, height)
        List of lists of the Text objects in the order they are plotted
        (lower left to upper right). Useful for putting annotations relative
        to specific objects in the motif.
    bbox : [[x0, y0], [x1, y1]]
        Full extent of the logo in screen coodinates.
    bbox_data : [[x0, y0], [x1, y1]]
        Full extent of the logo in data coodinates."""
    colors = getattr(aacolors, color_scheme)
    if axh is None:
        axh = plt.gca()
  
    trans_offset = transforms.offset_copy(axh.transData, 
                                      fig=axh.figure, 
                                      x=0, y=0, 
                                      units='dots')
    bottom_offset = trans_offset
    neg_trans_offset = trans_offset
    first_pos = None
    last_neg = None
    bbox = np.array([[x, y], [x, y]])
    mxy = y 
    mny = y
    letters = [[] for i in range(motif.shape[1])]
    for xi in range(motif.shape[1]):
        scores = motif.iloc[:, xi]
        pos_scores = scores[scores > 0].sort_values()
        neg_scores = (-scores[scores < 0]).sort_values()
        yshift = 0
        neg_trans_offset = trans_offset
        for yi, (aa, score) in enumerate(pos_scores.items()):
            txt = axh.text(x, 
                          y, 
                          aa, 
                          transform=trans_offset,
                          fontsize=fontsize, 
                          color=colors.color(aa),
                          ha='left',
                          family='monospace')
            txt.set_path_effects([Scale(1.5, score)])
            axh.figure.canvas.draw()
            window_ext = txt.get_window_extent(txt._renderer)
            
            #bbox_data = axh.transData.inverted().transform(np.array([[window_ext.x0, window_ext.y0], [window_ext.x1, window_ext.y1]]))
            #bbox_data = window_ext.inverse_transformed(axh.transData)
            #plt.plot(bbox_data.corners()[:, 0], bbox_data.corners()[:, 1], '-k')

            bbox = _extend_bbox(bbox, window_ext)
            letters[xi].append((txt, window_ext.height * score))
            if first_pos is None:
                first_pos = window_ext

            if yshift == 0:
                bottom_offset = transforms.offset_copy(txt._transform, 
                                                      fig=axh.figure,
                                                      x=window_ext.width * 1.5,
                                                      units='dots')

            yshift += window_ext.height * score
            
            # print(xi, aa, window_ext.x0, window_ext.y0, '%1.1f' % score, '%1.0f' % window_ext.height, '%1.0f' % window_ext.width, '%1.1f' % yshift)
            trans_offset = transforms.offset_copy(txt._transform, 
                                                  fig=axh.figure,
                                                  y=window_ext.height * score,
                                                  units='dots')
        trans_offset = bottom_offset
        if yshift > mxy:
            mxy = yshift

    trans_offset = transforms.offset_copy(axh.transData, 
                                          fig=axh.figure, 
                                          x=0, y=0, 
                                          units='dots')    
    for xi in range(motif.shape[1]):
        scores = motif.iloc[:, xi]
        pos_scores = scores[scores > 0].sort_values()
        neg_scores = (-scores[scores < 0]).sort_values()
        yshift = 0
        for yi, (aa, score) in enumerate(neg_scores.items()):
            txt = axh.text(x, 
                          y, 
                          aa, 
                          transform=trans_offset,
                          fontsize=fontsize, 
                          color=colors.color(aa),
                          ha='left',
                          va='top',
                          family='monospace')
            txt.set_path_effects([Scale(1.5, -score)])
            axh.figure.canvas.draw()
            window_ext = txt.get_window_extent(txt._renderer)
            bbox = _extend_bbox(bbox, window_ext)
            letters[xi].append((txt, window_ext.height * score))
            last_neg = window_ext

            if yshift == 0:
                bottom_offset = transforms.offset_copy(txt._transform, 
                                                      fig=axh.figure,
                                                      x=window_ext.width * 1.5,
                                                      units='dots')
            yshift -= window_ext.height * score
            
            # print(xi, aa, '%1.1f' % score, '%1.0f' % window_ext.height, '%1.0f' % window_ext.width, '%1.1f' % yshift)
            trans_offset = transforms.offset_copy(txt._transform, 
                                                      fig=axh.figure,
                                                      y=-window_ext.height * score,
                                                      units='dots')
        trans_offset = bottom_offset
        if yshift < mny:
            mny = yshift
        

    if last_neg is None:
        last_neg = first_pos
    plt.show()
    
    #bbox = np.array([[first_pos.x0, last_neg.y0], [window_ext.x1, window_ext.y0 + window_ext.height * score]])
    bbox_data = axh.transData.inverted().transform(bbox)
    return letters, bbox, bbox_data




import pandas as pd
import numpy as np
import warnings
from scipy import stats
from scipy.stats import multinomial
# import statsmodels.api as sm

try:
    import parasail
except ImportError:
    print('Importing palmotif without alignment support from parasail.')

__all__ = ['compute_motif',
           'compute_pal_motif',
           'uniprot_frequency',
           'compute_relative_motif',
           'consensus_seq']

aa_alphabet = [aa for aa in 'ARNDCQEGHILKMFPSTWYVBZX-']

uniprot_frequency = {'A': 8.25,
                     'R': 5.53,
                     'N': 4.06,
                     'D': 5.45,
                     'C': 1.37,
                     'Q': 3.93,
                     'E': 6.75,
                     'G': 7.07,
                     'H': 2.27,
                     'I': 5.96,
                     'L': 9.66,
                     'K': 5.84,
                     'M': 2.42,
                     'F': 3.86,
                     'P': 4.70,
                     'S': 6.56,
                     'T': 5.34,
                     'W': 1.08,
                     'Y': 2.92,
                     'V': 6.87}

def compute_motif(seqs, reference_freqs=None, weights=None, align_first=False, gap_reduce=None, alphabet=None, oddsratio=False, pseudo_count=0):
    """Compute heights for a sequence logo based on the frequency of AAs/symbols at each position of the
    aligned sequences. The output matrix/DataFrame contains the relative entropy of each AA/symbol in bits. It
    is computed relative to uniform frequencies (default) or some fixed set of frequencies (not position specific).

    The output is often used as symbol heights in a logo plot. Heights would reflect the fraction of total
    entropy contributed by that AA within each column of the alignment.

    Sequences must be pre-aligned using a tool like ClustalW, MUSCLE or other tool for multiple-sequence alignment.

    Parameters
    ----------
    seqs : list or pd.Series
        Alignment of AA sequences
    reference_freqs : dict or None
        Reference frequencies for each AA
    weights : np.array
        Weights for each sequence
    gap_reduce : None or float threshold
        Identifies columns with threshold fraction of gaps,
        removes the sequences that have AAs there and
        removes the column from the alignment.
    alphabet : str or list
        Limit frequencies to only a subset of AAs
        Default will consider all amino-acids.

    Returns
    -------
    motif : pd.DataFrame
        Heights for each symbol (index, rows) for each column.
        Heights reflect the fraction of total entropy contributed
        by that AA within each column of the alignment."""

    if alphabet is None:
        alphabet = aa_alphabet

    if weights is None:
        weights = np.ones(len(seqs))
    
    if not isinstance(seqs, pd.Series):
        align = pd.Series(seqs)
    else:
        align = seqs

    if not gap_reduce is None:
        align = reduce_gaps(align, thresh=gap_reduce)

    L = len(align.iloc[0])
    
    freq = _get_frequencies(align, alphabet, weights, pseudo_count=pseudo_count)
    
    # red_alphabet = freq.index[(np.abs(freq) > 0).any(axis=1)]
    nAA = len(alphabet)

    with warnings.catch_warnings():
        warnings.simplefilter('ignore')

        if not reference_freqs is None:
            min_freq = np.min([f for f in reference_freqs.values()])
            reference_freqs = np.array([reference_freqs.get(aa, min_freq / 2) for aa in alphabet])
            reference_freqs = reference_freqs / np.sum(reference_freqs)
            reference_freqs = np.tile(reference_freqs[:, None], (1, L))
            if oddsratio:
                # heights = reference_freqs * np.log2(freq / reference_freqs)
                heights = np.log2(freq / reference_freqs)
            else:    
                heights = freq * np.log2(freq / reference_freqs)
            heights[np.isnan(heights)] = 0
        else:
            tot_entropy = np.log2(nAA) - np.nansum(-freq * np.log2(freq), axis=0, keepdims=True)
            heights = freq * tot_entropy

    return pd.DataFrame(heights, columns=list(range(L)), index=alphabet)

def reduce_gaps(align, thresh=0.7):
    """Identifies columns with thresh fraction of gaps,
    removes the sequences that have AAs there and
    removes the column from the alignment

    Parameters
    ----------
    align : pd.Series
        Contains one row per sequence, with each sequence assumed to
        have the same length and be aligned
    thresh : fraction
        Positions with more than thresh fraction of sequences having
        a gap character will be removed from the alignment and the 1 - thresh
        sequences that don't have a gap at that position will be removed
        from the aligmment

    Returns
    -------
    out : pd.Series
        Similar to input align, but with possibly fewer sequences (rows)
        and shorted sequences."""
    removePos = []
    for pos in range(len(align.iloc[0])):
        if np.mean(align.map(lambda seq: seq[pos] == '-')) > thresh:
            removePos.append(pos)
    for pos in removePos:
        align = align.loc[align.map(lambda seq: seq[pos] == '-')]
    return align.map(lambda seq: ''.join([aa for pos, aa in enumerate(seq) if not pos in removePos]))

def pairwise_alignment_frequencies(centroid, seqs, gopen=3, gextend=3, matrix=parasail.blosum62, alphabet=None):
    """Aligns each seq in seqs to centroid using global Needleman-Wunsch and then counts
    the AA/symbols in each seq at each alignment/centroid position. Alignment positions
    containing a gap/"-" in the centroid sequence are not counted.

    Parameters
    ----------
    centroid : str
        Sequence to align against. It does not need to be the centroid, though that
        would be a sensible choice.
    seqs : iterable
        List of sequences of varying length to be globally aligned to centroid seq
    gopen, gextend, matrix : alignment parameters for parasail

    Returns
    -------
    counts : np.ndarray [position x symbol x seq]
        A binary matrix indicating the presence of a symbol after alignment
        for each position of the centroid, each symbol and each sequence."""
    if alphabet is None:
        alphabet = aa_alphabet

    centroid_query = parasail.profile_create_stats_sat(centroid, matrix=matrix)
    
    seq_algn = np.zeros((len(centroid), len(alphabet), len(seqs)))
    for seqi, s in enumerate(seqs):
        # a = parasail.nw_trace(centroid, s, open=3, extend=3, matrix=parasail.blosum62)
        a = parasail.nw_trace_scan_profile_sat(centroid_query, s, open=gopen, extend=gextend)
        
        pos = 0
        for ref_aa, q_aa in zip(a.traceback.ref, a.traceback.query):
            if not q_aa == '-':
                seq_algn[pos, alphabet.index(ref_aa), seqi] = 1
                pos += 1
    return seq_algn

def _get_frequencies(seqs, alphabet, weights, pseudo_count=0):
    L = len(seqs[0])
    nAA = len(alphabet)
    
    freq = np.zeros((nAA, L))
    for coli in range(L):
        for si, s in enumerate(seqs):
            freq[alphabet.index(s[coli]), coli] += weights[si]
    freq = (freq + pseudo_count) / (freq + pseudo_count).sum(axis=0, keepdims=True)
    return freq

def compute_relative_motif(seqs, refs, alphabet=None, oddsratio=False, pseudo_count=1):
    """Use statistic related to the Kullback-Liebler divergence to indicate how surprising
    it is to see the AAs in seqs vs. the refs.
    All seqs and refs must have the same length (i.e. be "aligned")
    
    For discussion about relative logos:
    https://bmcbioinformatics.biomedcentral.com/articles/10.1186/s12859-018-2489-3
    https://academic.oup.com/nar/article/40/W1/W281/1076274

    Parameters
    ----------
    seqs : iterable
        List of AA sequences that are all similar to each other and the centroid
    refs : iterable
        List of all other sequences in the experiment as a reference.

    Returns
    -------
    A : pd.DataFrame [AA alphabet x position]
        A matrix of KL scores with a higher divergence
        being more surprising."""

    """
    p_i is reference
    q_i is observed

    A = q * np.log2(q / p)

    """
    """Adding 1 to the refs to avoid inf's, but this should be studied more carefully"""
    if alphabet is None:
        alphabet = aa_alphabet

    p = _get_frequencies(refs, alphabet, np.ones(len(refs)), pseudo_count=pseudo_count)
    q = _get_frequencies(seqs, alphabet, np.ones(len(seqs)))
    with warnings.catch_warnings():
        warnings.simplefilter('ignore')
        if oddsratio:
            A = np.log2(q/p)
        else:
            A = q * np.log2(q/p)
    A[np.isnan(A)] = 0
    A = pd.DataFrame(A, index=alphabet)
    return A

def compute_pal_motif(centroid, seqs, refs=None,
                      gopen=3, gextend=3, matrix=None,
                      ref_freqs=None, alphabet=None, bootstrap_samples=0, alpha=0.05,
                      pseudo_count=1, oddsratio=False):
    """Compute pairwise alignments between the centroid and all sequences in seqs and refs. The motif
    will have the same length as the centroid with a statistic for each symbol at each position
    indicating the divergence of the observed distribution from the reference. Summing the statistics
    across the alphabet gives the Kullback-Liebler divergence for each position.

    Also returns the position specific log-likliehood of the sequences,
    given the reference uing a multinomial distribution on the alphabet.

    A 95% confidence interval on the output can be optionally computed with a
    non-parametric bootstrap. Recommend 10K samples for good estimation.

    Parameters
    ----------
    centroid : str
        Amino-acid sequence that is also among the seqs
    seqs : list
        List of AA sequences that are all similar to each other and the centroid
    refs : list
        List of all other sequences in the experiment as a reference.
    gopen : int
        Gap open penalty for parasail
    gextend : int
        Gap extend penalty for parasail
    alphabet : str
        String of characters whose frequency should be considered/counted
    matrix : substitution matrix
        Matrix from parasail for the alignment
        None for default parasail.blosum62
    ref_freqs : dict or None
        Reference frequencies for each AA
    pseudo_count : float
        A pseudo_count added to each alphabet symbol's count in the reference set,
        to "shrink" the reference probabilities toward uniform and prevent infinite
        outputs.
    bootstrap_samples : bool
        With values > 0, compute percentile bootstrap confidence intervals for
        the position-specific KL and loglikelihood statistics.

    Returns
    -------
    A : pd.DataFrame [AA alphabet x position]
        A matrix of KL divergence scores that can be used directly with the plotPALLogo function
    out : pd.DataFrame [position x (KL, loglikelihood columns)]
        Per position KL divergence from, and log-likelihood of observing the sequences, given the reference.
    """
    if matrix is None:
        matrix = parasail.blosum62
    
    if alphabet is None:
        alphabet = aa_alphabet

    """Returned DataFrame is [positions x alphabet] opposite of output"""
    raw_seq_algn = pairwise_alignment_frequencies(centroid, seqs, gopen=gopen, gextend=gextend, matrix=matrix, alphabet=alphabet)
    seq_algn = pd.DataFrame(raw_seq_algn.sum(axis=2), index=list(centroid), columns=alphabet)
    L = seq_algn.shape[0]

    if refs is None:
        if ref_freqs is None:
            p = np.ones(len(aa_alphabet))
        else:
            p = np.array([ref_freqs.get(aa, 0) for aa in aa_alphabet])
        p = np.tile(p[None, :], (L, 1))
        p = p / np.sum(p, axis=1, keepdims=True)
    else:
        raw_ref_algn = pairwise_alignment_frequencies(centroid, refs, gopen=gopen, gextend=gextend, matrix=matrix)
        ref_algn = pd.DataFrame(raw_ref_algn.sum(axis=2), index=list(centroid), columns=alphabet)
        """Adding 1 to refs to avoid inf's, but this should be studied more carefully
        https://en.wikipedia.org/wiki/Additive_smoothing"""
        p = (ref_algn.values + pseudo_count) / (ref_algn.values + pseudo_count).sum(axis=1, keepdims=True)

    """
    Similar to the Kullback-Liebler divergence between
    the sequences and the reference. Sum the symbols at
    each position to get KL divergence for that position.
    Sum across positions to get overal KL divergence.
    p_i is reference
    q_i is observed

    A = q * np.log2(q / p)
    """
    q = (seq_algn.values) / (seq_algn.values).sum(axis=1, keepdims=True)
    with warnings.catch_warnings():
        warnings.simplefilter('ignore')
        if oddsratio:
            A = np.log2(q/p)
        else:
            A = q * np.log2(q/p)
        kl = _kl_stat(np.sum(raw_seq_algn, axis=2), p)
    A[np.isnan(A)] = 0
    A = pd.DataFrame(A, index=seq_algn.index, columns=seq_algn.columns)
        
    kl_samps = np.zeros((len(centroid), bootstrap_samples))

    loglik = _multinomial_loglik(np.sum(raw_seq_algn, axis=2), p)
    loglik_samps = np.zeros((len(centroid), bootstrap_samples))
    
    with warnings.catch_warnings():
        warnings.simplefilter('ignore')
        for sampi in range(bootstrap_samples):
            rind = np.random.randint(raw_seq_algn.shape[2], size=raw_seq_algn.shape[2])
            loglik_samps[:, sampi] = _multinomial_loglik(np.sum(raw_seq_algn[:, :, rind], axis=2), p)
            kl_samps[:, sampi] = _kl_stat(np.sum(raw_seq_algn[:, :, rind], axis=2), p)
    if bootstrap_samples:
        kl_samps.sort(axis=1)
        loglik_samps.sort(axis=1)
        nvals = np.round((bootstrap_samples - 1) * np.array([alpha, 1-alpha])).astype(int)
        out = dict(kl=kl,
                    kl_ll=kl_samps[:, nvals[0]],
                    kl_ul=kl_samps[:, nvals[1]],
                    loglik=loglik,
                    loglik_ll=loglik_samps[:, nvals[0]],
                    loglik_ul=loglik_samps[:, nvals[1]])
    else:
        out = dict(kl=kl, loglik=loglik)
    out = pd.DataFrame(out, index=list(centroid))
    return A.T, out

def _kl_stat(counts, ref_pr):
    q = counts / np.sum(counts, axis=1, keepdims=True)
    return np.nansum(q * np.log2(q / ref_pr), axis=1)

def _multinomial_loglik(counts, ref_pr):
    """A and seq_algn have alphabet in columns (before A gets transposed)"""
    loglik = np.zeros(counts.shape[0])
    for posi in range(counts.shape[0]):
        loglik[posi] = multinomial.logpmf(counts[posi, :], np.sum(counts[posi, :]), ref_pr[posi, :])
    return loglik

def consensus_seq(motif):
    """With a motif pd.DataFrame, the symbol with the highest value at each
    position is the "consensus" sequence."""
    return ''.join(motif.idxmax(axis=0).values)

def multinomial_confint(counts, alpha=0.05):
    """Confidence interval on observed counts from a multinomial distribution.
    Code for the Goodman (1965) method is lifted from statsmodels 0.12
    which was not currently released.

    License for statsmodels package is below. This function will be replaced by
    a dependency on statsmodels 0.12 upon release.

    Copyright (C) 2006, Jonathan E. Taylor
    All rights reserved.

    Copyright (c) 2006-2008 Scipy Developers.
    All rights reserved.

    Copyright (c) 2009-2018 statsmodels Developers.
    All rights reserved.


    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:

      a. Redistributions of source code must retain the above copyright notice,
         this list of conditions and the following disclaimer.
      b. Redistributions in binary form must reproduce the above copyright
         notice, this list of conditions and the following disclaimer in the
         documentation and/or other materials provided with the distribution.
      c. Neither the name of statsmodels nor the names of its contributors
         may be used to endorse or promote products derived from this software
         without specific prior written permission.


    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED. IN NO EVENT SHALL STATSMODELS OR CONTRIBUTORS BE LIABLE FOR
    ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
    DAMAGE."""

    n = counts.sum()
    k = len(counts)
    proportions = counts / n
    chi2 = stats.chi2.ppf(1 - alpha / k, 1)
    delta = chi2 ** 2 + (4 * n * proportions * chi2 * (1 - proportions))
    region = ((2 * n * proportions + chi2 +
               np.array([- np.sqrt(delta), np.sqrt(delta)])) /
              (2 * (chi2 + n))).T
    return region


seqs1 = ['AAAKILQTPK',
     'AAFDGGFTVK',
     'AALSTALSEK',
     'AASSHVPIQK',
     'AGNFHITVGK',
     'ALADAILINK',
     'ALADFLPVMK',
     'ALADIILINK',
     'AIADSPTGQK',
     'ALAEKFKELK',
     'ALALGVTLLK',
     'ALAPILLATL',
     'ALAPSTMKIK',
     'AIAQPATTAK',
     'ALASLVSAPK',
     'ALDENQRLAK',
     'ALDTKIGRYK',
     'ALFAKPLKGK',
     'ALFASGLIHR',
     'AIFFLPDSSK',
     'AIFHGWVLQK',
     'ALFKPGAWFK',
     'AIFSVYVVNK',
     'ALKNPPINTK',
     'ALKSINNAEK',
     'ALLAYTLGVK',
     'AIIDHIFASK',
     'ALIDYENSNK',
     'ALIEELLLYK',
     'AILEGDFSTK',
     'ALIGFPSVGK',
     'AILGYRSVGK',
     'ALLPAGFRQK',
     'ALIPGTLIKK',
     'ALLPWPFKQK',
     'ALLPWPFRNK',
     'ALIQQATTVK',
     'ALLSMTFPLK',
     'ALNDVTAARK',
     'AINEVVTREY',
     'AINPINTFTK',
     'ALNSAEKITK',
     'ALPSRILLWK',
     'AIQNPTMINK',
     'ALQSITAGQK',
     'ALRDNSTMGY',
     'ALRETGVIEK',
     'ALSAVSTQQK',
     'AISSFIAYQK',
     'ALSSIISRQK']

seqs2 = ['VVNDERFVSK',
 'VVNIVPVIAK',
 'VVPEMTEILK',
 'VVPKAAIVAR',
 'VVPKKLTIGK',
 'VVPLEEGLNK',
 'VVQEPGQVFK',
 'VVQTLVAKEK',
 'VVSESLTPER',
 'VVSSGPSVLR',
 'VVSSVVKVKK',
 'VVVDPIQSVK',
 'VVVFPAGLLQ',
 'VVVGAGGVGK',
 'VVVGDGAVGK',
 'VVVGNGAVGK',
 'VVVNNAGILR',
 'VVVPPLPGKA',
 'VVYEDIFDAK',
 'VVYGGKSTIR',
 'VVYPPPVDPK',
 'VVYTYTGKRI',
 'VVYVGGILTK',
 'VYGQKEIHRK',
 'VYPFPGPIPN',
 'WSSSRPIVQK',
 'YAFMGTPVQK',
 'YALPHAILRL',
 'YGSAPRTPSK',
 'YGYSNPKILK',
 'YKYIQELWRK',
 'YIIDFGLAKK',
 'YIQKNVQLYK',
 'YIQTDHLFFK',
 'YIYDGELVSK',
 'YSAFGVIIEK',
 'YSKEGWEYVK',
 'YSNYGPPQGK',
 'YSYGDFLLGK',
 'YSYQYTVANK',
 'YSYYNLILKK',
 'YTASLVGMIK',
 'YTCSFCGKTK',
 'YTFFTPGNLK',
 'YTFYSISQER',
 'YTLDPDTLRK',
 'YTLDVTTGQR',
 'YTIPSDFISK',
 'YTYEHDPITK',
 'YVLDDLIVAK']


"""Most are just paths with fill.
Second and third paths are white e.g. inside of A.
O and Q require circles and S requires stroke only.

Paths are derived from those found in logojs:
https://github.com/weng-lab/logojs-package

MIT License

Copyright (c) 2018 weng-lab

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE."""

__all__ = ['svg_letter',
           'add_letter']

alphabet_paths = {'A':['M 0 100 L 33 0 L 66 0 L 100 100 L 75 100 L 66 75 L 33 75 L 25 100 L 0 100',
                        'M 41 55 L 50 25 L 58 55 L 41 55'],
                  'B':['M 0 0 L 80 0 C 105 0 105 50 80 50 C 105 50 105 100 80 100 L 00 100 L 0 0',
                       'M 20 15 L 70 15 C 80 15 80 35 70 35 L 20 35 L 20 15',
                       'M 20 65 L 70 65 C 80 65 80 85 70 85 L 20 85 L 20 65'],
                  'C':['M 100 28 C 100 -13 0 -13 0 50 C 0 113 100 113 100 72 L 75 72 C 75 90 30 90 30 50 C 30 10 75 10 75 28 L 100 28'],
                  'D':['M 0 0 L 60 0 C 110 0 110 100 60 100 L 0 100 L 0 0',
                       'M 20 15 L 40 15 C 85 15 85 85 40 85 L 20 85 L 20 15'],
                  'E':['M 0 0 L 100 0 L 100 20 L 20 20 L 20 40 L 90 40 L 90 60 L 20 60 L 20 80 L 100 80 L 100 100 L 0 100 L 0 0'],
                  'F':['M 0 0 L 100 0 L 100 20 L 20 20 L 20 40 L 80 40 L 80 60 L 20 60 L 20 100 L 0 100 L 0 0'],
                  'G':['M 100 28 C 100 -13 0 -13 0 50 C 0 113 100 113 100 72 L 100 48 L 55 48 L 55 72 L 75 72 C 75 90 30 90 30 50 C 30 10 75 5 75 28 L 100 28'],
                  'H':['M 0 0 L 20 0 L 20 40 L 80 40 L 80 0 L 100 0 L 100 100 L 80 100 L 80 60 L 20 60 L 20 100 L 0 100 L 0 0'],
                  'I':['M 40 0 L 60 0 L 60 100 L 40 100 L 40 0'],
                  'J':['M 0 60 C 0 111 100 111 100 60 L 100 0 L 75 0 L 75 60 C 80 90 20 90 25 60'],
                  'K':['M 0 0 L 20 0 L 20 40 L 75 0 L 100 0 L 50 50 L 100 100 L 75 100 L 30 65 L 20 75 L 20 100 L 0 100 L 0 0'],
                  'L':['M 0 0 L 0 100 L 100 100 L 100 80 L 20 80 L 20 0 L 0 0'],
                  'M':['M 0 0 L 20 0 L 50 35 L 80 0 L 100 0 L 100 100 L 80 100 L 80 30 L 50 65 L 20 30 L 20 100 L 0 100 L 0 0'],
                  'N':['M 0 100 L 0 0 L 20 0 L 80 75 L 80 0 L 100 0 L 100 100 L 80 100 L 20 25 L 20 100 L 0 100'],
                  'O':['cx="50" cy="50" r="50"', 'cx="50" cy="50" r="32"'], # requires circles
                  'P':['M 0 0 L 80 0 C 105 0 105 50 80 50 L 20 50 L 20 100 L 0 100 L 0 0',
                       'M 20 15 L 70 15 C 80 15 80 35 70 35 L 20 35 L 20 15'],
                  'Q':['M 85 100 L 55 70 L 70 55 L 100 85 L 85 100', 'cx="50" cy="50" r="50"', 'cx="50" cy="50" r="32"'], # O with slash
                  'R':['M 0 0 L 80 0 C 105 0 105 50 80 50 C 100 50 100 70 100 70 L 100 100 L 80 100 L 80 80 C 80 80 80 60 50 60 L 20 60 L 20 100 L 0 100 L 0 0',
                        'M 20 15 L 70 15 C 80 15 80 35 70 35 L 20 35 L 20 15'],
                  'S':['M92 26 A43 20 0 1 0 43 46 A42 23 0 1 1 9 68'], #stroke only
                  'T':['M 0 0 L 0 20 L 35 20 L 35 100 L 65 100 L 65 20 L 100 20 L 100 0 L 0 0'],
                  'U':['M 0 0 L 0 60 C 0 111 100 111 100 60 L 100 0 L 75 0 L 75 60 C 80 90 20 90 25 60 L 25 0 L 0 0'],
                  'V':['M 0 0 L 20 0 L 50 80 L 80 0 L 100 0 L 60 100 L 40 100 L 0 0'],
                  'W':['M 0 0 L 20 0 L 30 70 L 50 30 L 70 70 L 80 0 L 100 0 L 90 100 L 70 100 L 50 65 L 30 100 L 10 100 L 0 0'],
                  'X':['M 0 0 L 20 0 L 50 40 L 80 0 L 100 0 L 70 50 L 100 100 L 80 100 L 50 60 L 20 100 L 0 100 L 30 50 L 0 0'],
                  'Y':['M 0 0 L 20 0 L 50 45 L 80 0 L 100 0 L 60 60 L 60 100 L 40 100 L 40 60 L 0 0'],
                  'Z':['M 0 0 L 100 0 L 100 20 L 35 80 L 100 80 L 100 100 L 0 100 L 0 80 L 65 20 L 0 20 L 0 0'],
                  '-':['M 12.5 40 H 87.5 V 60 H 12.5 V 40']}

def svg_letter(a, color='black', background='white'):
    paths_only = 'ABCDEFGHIJKLMNPRTUVWXYZ-'
    if a in paths_only:
        out = [{'element':'path', 'd':alphabet_paths[a][0], 'fill':color}]
        for ap in alphabet_paths[a][1:]:
            out.append({'element':'path', 'd':ap, 'fill':background})
    elif a == 'O':
        out = [{'element':'circle', 'center':(50, 50), 'r':50, 'fill':color},
               {'element':'circle', 'center':(50, 50), 'r':32, 'fill':background}]
    elif a == 'Q':
        out = [{'element':'circle', 'center':(50, 50), 'r':50, 'fill':color},
               {'element':'circle', 'center':(50, 50), 'r':32, 'fill':background},
               {'element':'path', 'd':'M 85 100 L 55 70 L 70 55 L 100 85 L 85 100', 'fill':color}]
    elif a == 'S':
        out = [{'element':'path', 'd':'M92 26 A43 20 0 1 0 43 46 A42 23 0 1 1 9 68', 'stroke':color, 'fill':background, 'stroke-width':18}]
    return out

def add_letter(dwg, letter, group_id='', color='black', background='white', **attributes):
    if group_id is None:
        group_id = 'letter_%s' % letter
    elements = svg_letter(letter, color=color, background=background)
    group = dwg.add(dwg.g(id=group_id, **attributes))
    for e in elements:
        if e['element'] == 'path':
            tmp_attrib = {k:e[k] for k in e if not k == 'element'}
            group.add(dwg.path(**tmp_attrib))
        elif e['element'] == 'circle':
            tmp_attrib = {k:e[k] for k in e if not k == 'element'}
            group.add(dwg.circle(**tmp_attrib))
    return group





"""Color schemes derived from those in weblogo:
https://github.com/ostrokach/weblogo/blob/master/weblogolib/colorscheme.py"""

#  Copyright (c) 2003-2005 The Regents of the University of California.
#  Copyright (c) 2005 Gavin E. Crooks

#  This software is distributed under the MIT Open Source License.
#  <http://www.opensource.org/licenses/mit-license.html>
#
#  Permission is hereby granted, free of charge, to any person obtaining a 
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
#  THE SOFTWARE.

""" Popular color codings for nucleic and amino acids. 
Classes:
    ColorScheme -- A color scheme
    ColorGroup  
    
    
Generic
    monochrome
Nucleotides
    nucleotide
    base pairing
Amino Acid
    hydrophobicity
    chemistry
    charge
    taylor
Status : Beta - Needs documentation.
"""
# Good online references include bioruby and the JalView alignment editor.
# Clamp, M., Cuff, J., Searle, S. M. and Barton, G. J. (2004), 
# "The Jalview Java Alignment Editor," Bioinformatics, 12, 426-7
# http://www.jalview.org

import pandas as pd
import io

__all__ = ['aa_grouping',
           'color_group',
           'nucleotide',
           'base_pairing',
           'hydrophobicity',
           'chemistry',
           'charge',
           'taylor',
           'logojs',
           'shapely',
           'black']

class aa_grouping(object):
    """Builds lookup dicts of color and label assignments from the set of color_groups"""
    def __init__(self, groups, title='', description=''):
        self.groups = groups
        self.labelDict = {}
        self.colorDict = {}
        for g in groups:
            self.colorDict.update({aa:g.color for aa in g.aas})
            self.labelDict.update({aa:g.label for aa in g.aas})

    def label(self, aa):
        return self.labelDict.get(aa, '')

    def color(self, aa):
        return self.colorDict.get(aa, 'black')

class color_group(dict):
    """Builds a lookup dict for the amino-acids specified"""
    def __init__(self, aas, color, label=''):
        dict.__init__(self)
        self.color = color
        self.label = label
        self.aas = aas

black = aa_grouping([])

nucleotide = aa_grouping([
    color_group("G", "orange"),
    color_group("TU", "red"),
    color_group("C",  "blue"),
    color_group("A",  "green")]) 

base_pairing = aa_grouping([
    color_group("TAU",  "darkorange", "Weak (2 Watson-Crick hydrogen bonds)"),
    color_group("GC",    "blue", "Strong (3 Watson-Crick hydrogen bonds)")])

# From Crooks2004c-Proteins-SeqStr.pdf
hydrophobicity = aa_grouping([
    color_group( "RKDENQ",   "blue", "hydrophilic"),
    color_group( "SGHTAP",   "green", "neutral"  ),
    color_group( "YVMCLFIW", "black",  "hydrophobic") ])

# from makelogo
chemistry = aa_grouping([
  color_group( "GSTYC",  "green",   "polar"),
  color_group( "NQ",      "purple", "neutral"), 
  color_group( "KRH",     "blue",   "basic"),
  color_group( "DE",      "red",    "acidic"),
  color_group("PAWFLIMV", "black",  "hydrophobic") ])   

charge = aa_grouping([
    color_group("KRH", "blue", "Positive" ),
    color_group( "DE", "red", "Negative") ])


taylor = aa_grouping([
    color_group( 'A', '#CCFF00' ),
    color_group( 'C', '#FFFF00' ),
    color_group( 'D', '#FF0000'),
    color_group( 'E', '#FF0066' ),
    color_group( 'F', '#00FF66'),
    color_group( 'G', '#FF9900'),
    color_group( 'H', '#0066FF'),
    color_group( 'I', '#66FF00'),
    color_group( 'K', '#6600FF'),
    color_group( 'L', '#33FF00'),
    color_group( 'M', '#00FF00'),
    color_group( 'N', '#CC00FF'),
    color_group( 'P', '#FFCC00'),
    color_group( 'Q', '#FF00CC'),
    color_group( 'R', '#0000FF'),
    color_group( 'S', '#FF3300'),
    color_group( 'T', '#FF6600'),
    color_group( 'V', '#99FF00'),
    color_group( 'W', '#00CCFF'),
    color_group( 'Y', '#00FFCC')],
    title = "Taylor",
    description = "W. Taylor, Protein Engineering, Vol 10 , 743-746 (1997)")

_logojs_colors = {'A': '#000000',
                 'B': '#bb8800',
                 'C': '#008811',
                 'D': '#ff0000',
                 'E': '#ff0022',
                 'F': '#333333',
                 'G': '#007700',
                 'H': '#220099',
                 'I': '#111111',
                 'K': '#0000aa',
                 'L': '#002222',
                 'M': '#220022',
                 'N': '#009911',
                 'P': '#080808',
                 'Q': '#00aa00',
                 'R': '#0022aa',
                 'S': '#008f00',
                 'T': '#006600',
                 'V': '#222200',
                 'W': '#080808',
                 'Y': '#00a800',
                 'Z': '#aaaa00',
                 '-': '#000000'}
"""Amino acids colored according to chemical properties (acidic, basic, and non-polar are red, blue, and black, respectively"""
logojs = aa_grouping([color_group(aa, color) for aa, color in _logojs_colors.items()],
                     title='LogoJS',
                     description='https://github.com/weng-lab/logojs-package')


_shapely_colors = 'Amino Acids,Color Name,RGB Values,Hexadecimal\nD|E,bright red,"[230,10,10]",E60A0A\nC|M,yellow,"[230,230,0]",E6E600\nK|R,blue,"[20,90,255]",145AFF\nS|T,orange,"[250,150,0]",FA9600\nF|Y,mid blue,"[50,50,170]",3232AA\nN|Q,cyan,"[0,220,220]",00DCDC\nG,light grey,"[235,235,235]",EBEBEB\nL|V|I,green,"[15,130,15]",0F820F\nA,dark grey,"[200,200,200]",C8C8C8\nW,pink,"[180,90,180]",B45AB4\nH,pale blue,"[130,130,210]",8282D2\nP,flesh,"[220,150,130]",DC9682'
_colors_df = pd.read_csv(io.StringIO(_shapely_colors), delimiter=',')
shapely = aa_grouping([color_group(r['Amino Acids'].replace('|', ''), '#%s' % r['Hexadecimal']) for i,r in _colors_df.iterrows()],
                      title='Shapely',
                      description='http://acces.ens-lyon.fr/biotic/rastop/help/colour.htm#shapelycolours')



import numpy as np
import pandas as pd
import svgwrite
import matplotlib as mpl

from .svg_alphabet import add_letter
from . import aacolors

__all__ = ['svg_logo']

"""Note for converting SVG to PNG with imagemagick:
import subprocess

cmd = ['convert', '-density 200', 'test.svg', 'test.png']
subprocess.call(' '.join(cmd), shell=True)"""

_basic_html = """<html>
<svg width="{width}" height="{height}">
{svg}
</svg>
</html>"""

def svg_logo(motif, filename=None,
             color_scheme='chemistry',
             return_str=False, return_html=False,
             yscale=1, svg_height='500px', svg_width='500px',
             ylabel='',
             yrange=None):
    """Sequence logo of the motif using SVG.

    Parameters
    ----------
    motif : pd.DataFrame
        Matrix of scores for each AA symbol (index, rows) and each
        column in an alignment (columns). Values will be used to scale
        each symbol linearly.
    filename : path
        File for saving the SVG.
    color_scheme : str
        An aa_grouping instance from aacolors.py or custom define.
        Options: nucleotide, base_pairing, hydrophobicity, chemistry, charge, taylor, logojs, shapely
    return_str : bool
        Optionally return SVG as text instead of saving to filename."""
    if filename is None:
        return_str = True
    if return_html:
        return_str = True

    colors = getattr(aacolors, color_scheme)

    yscale *= 4
    margin = 10
    left_margin = 100
    bottom_margin = 50
    xpad = 4
    HW = 100
    fontsize='20pt'
    ylabel_fontsize='28pt'

    if yrange is None:
        """Scale height of 100px to absolute max value"""
        # mx_value = np.max(np.abs(motif.values))
        mx_value = np.max(np.sum(np.abs(motif.values), axis=0))
        hscale = HW / mx_value
        wscale = 1

        mx_pos = 0
        mn_neg = 0
        for j in range(motif.shape[1]):
            tmp = motif.values[:, j]
            tot = np.sum(tmp[tmp>0])
            if tot > mx_pos:
                mx_pos = tot
            tot = np.sum(tmp[tmp<0])
            if tot < mn_neg:
                mn_neg = tot
    else:
        mn_neg, mx_pos = yrange
        mx_value = mx_pos - mn_neg
        hscale = HW / mx_value
        wscale = 1
        
    
    yticklabels = mpl.ticker.MaxNLocator(nbins=5, steps=[1, 2, 2.5, 5, 10]).tick_values(mn_neg, mx_pos)
    yticks = [hscale * yt * yscale for yt in yticklabels]

    mx_pos = hscale * mx_pos
    mn_neg = hscale * mn_neg

    mx_pos = np.max([mx_pos, yticks[-1]])
    mn_neg = np.min([mn_neg, yticks[0]])

    yzero = margin + mx_pos
    xzero = left_margin

    height = mx_pos - mn_neg + margin + bottom_margin
    width = wscale * (HW + xpad) * motif.shape[1] + margin + left_margin + xpad

    xticks = [xzero + (i + 1) * wscale * (HW + xpad) - wscale * HW / 2 for i in range(motif.shape[1])]
    # xticklabels = ['%d' % (i + 1) for i in range(motif.shape[1])]
    xticklabels = motif.columns

    # dwg = svgwrite.Drawing(filename=filename, height=height, width=width)
    # dwg = svgwrite.Drawing(filename=filename, size=(width, height), viewBox='0 0 %f %f' % (width, height))
    dwg = svgwrite.Drawing(filename=filename, size=('100%', '100%'), viewBox='0 0 %f %f' % (width, height))

    letter_groups = {}
    for xi in range(motif.shape[1]):
        xshift = xzero + xi*(HW + xpad) + xpad
        scores = motif.iloc[:, xi]
        pos_scores = scores[scores > 0].sort_values()
        neg_scores = (-scores[scores < 0]).sort_values()
        
        posshift = 2
        for yi, (aa, score) in enumerate(pos_scores.items()):
            scaled_height = hscale * score * yscale
            translate = (xshift, yzero - posshift - scaled_height)
            transform = 'translate({xtrans} {ytrans}) scale({xscale} {yscale})'.format(xtrans=translate[0], ytrans=translate[1],
                                                                                        xscale=wscale, yscale=yscale * score/mx_value)
            letter_groups[(xi, aa)] = add_letter(dwg, aa, group_id='%d_%s' % (xi, aa), color=colors.color(aa), background='white', transform=transform)
            #box = dwg.add(dwg.rect(insert=(x*cm, y*cm), size=(X*cm, score*fontsize*cm), fill='green', opacity=score))
            #print(aa, x, y, score)
            posshift += scaled_height

        negshift = 2
        for yi, (aa, score) in enumerate(neg_scores.items()):
            scaled_height = hscale * score * yscale
            translate = (xshift, yzero + negshift)
            transform = 'translate({xtrans} {ytrans}) scale({xscale} {yscale})'.format(xtrans=translate[0], ytrans=translate[1],
                                                                                        xscale=wscale, yscale=yscale * score/mx_value)
            letter_groups[(xi, aa)] = add_letter(dwg, aa, group_id='%d_%s' % (xi, aa), color=colors.color(aa), background='white', transform=transform)
            #box = dwg.add(dwg.rect(insert=(x*cm, y*cm), size=(X*cm, score*fontsize*cm), fill='green', opacity=score))
            #print(aa, x, y, score)
            negshift += scaled_height
    
    axes = dwg.add(dwg.g(id='axes', stroke_width=1.5, stroke='#000000'))
    axes.add(dwg.path(d='M {xz} {yz} h {len}'.format(xz=xzero, yz=yzero, len=wscale * (HW + xpad) * motif.shape[1] + 2 * xpad)))
    axes.add(dwg.path(d='M {xz} {yz} v {len}'.format(xz=xzero, yz=yzero - np.min(yticks), len=-(yticks[-1] - yticks[0]))))
    for yt, ytl in zip(yticks, yticklabels):
        axes.add(dwg.path(d='M {xz} {yz} h {len}'.format(xz=xzero, yz=yzero - yt, len=-(margin / 2))))
        axes.add(dwg.text(ytl, (xzero - (margin/2) - xpad, yzero - yt),
                          fill='#000000',
                          font_size=fontsize,
                          font_family='sans-serif',
                          font_weight='normal',
                          text_anchor='end',
                          dominant_baseline='middle'))
    for xt, xtl in zip(xticks, xticklabels):
        axes.add(dwg.text(xtl, (xt, yzero - yticks[0] + 10),
                          fill='#000000',
                          font_size=fontsize,
                          font_family='sans-serif',
                          font_weight='normal',
                          text_anchor='middle',
                          dominant_baseline='hanging'))
    x, y = (0, yzero - yticks[len(yticks) // 2])
    axes.add(dwg.text(ylabel, (x, y),
                          fill='#000000',
                          font_size=ylabel_fontsize,
                          font_family='sans-serif',
                          font_weight='normal',
                          text_anchor='middle',
                          dominant_baseline='hanging',
                          transform=f"rotate(-90, {x}, {y})"))

    if not return_str:
        dwg.save()
        return filename
    else:
        svg = dwg.tostring()
        if return_html:
            return _basic_html.format(svg=svg, height=svg_height, width=svg_width)
        else:
            return svg




from setuptools import setup, find_packages
PACKAGES = find_packages()

"""Read the contents of your README file"""
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

opts = dict(name='palmotif',
            maintainer='Andrew Fiore-Gartland',
            maintainer_email='agartlan@fredhutch.org',
            description='Python package for computing and plotting sequence logos, with output as SVG or matplotlib',
            long_description=(""""""),
            long_description_content_type='text/markdown',
            url='https://github.com/agartland/palmotif',
            license='MIT',
            author='Andrew Fiore-Gartland',
            author_email='agartlan@fredhutch.org',
            version='0.4',
            packages=PACKAGES
           )

requires = ['matplotlib',
            'numpy>=1.16',
            'pandas>=0.24.2',
            'parasail>=1.1.17',
            'svgwrite',
            'scipy>=1.4.1']
setup(**opts, install_requires=requires)



"""
python -m unittest palmotif/tests/test_plot.py
"""
import unittest
import numpy as np
import pandas as pd
from os.path import join as opj

from palmotif import *

from palmotif.tests.sequences import seqs1, seqs2


class TestPalmotif(unittest.TestCase):
    def test_svg_plot(self):
        fn = opj('palmotif', 'tests', 'test.svg')
        a = pd.DataFrame(np.array([[0.1, 0.2, 0, 0.5],[0, 0.3, 0.8, 0.1], [0.5, 0.1, 0.4, 0.]]).T, columns=[0, 1, 2], index=['A', 'C', 'T', 'G'])
        svg_logo(a, fn, color_scheme='nucleotide')
        self.assertTrue(True)
    
    def test_svg_negative(self):
        fn = opj('palmotif', 'tests', 'negative.svg')
        a = pd.DataFrame(np.array([[0.1, -0.2, 0, 0.5],[0, 0.3, -0.8, 0.1]]).T, columns=[0, 1], index=['C', 'R', 'W', 'S'])
        svg_logo(a, fn)
        self.assertTrue(True)
    
    def test_svg_alphabet(self):
        fn = opj('palmotif', 'tests', 'alphabet.svg')
        aas = [aa for aa in 'ABCDEFGHIKLMNPQRSTVWXYZ']

        np.random.seed(110820)
        a = pd.DataFrame(np.log10(10*np.random.rand(len(aas), 10)), columns=['P%d' % (i + 1) for i in range(10)], index=aas)
        svg_logo(a, fn)
        self.assertTrue(True)

    def test_svg_taylor(self):
        fn = opj('palmotif', 'tests', 'taylor.svg')
        aas = [aa for aa in 'ABCDEFGHIKLMNPQRSTVWXYZ']

        np.random.seed(110820)
        a = pd.DataFrame(np.log10(10*np.random.rand(len(aas), 10)), columns=['P%d' % (i + 1) for i in range(10)], index=aas)
        svg_logo(a, fn, color_scheme='taylor')
        self.assertTrue(True)

    def test_compute_motif(self):
        motif = compute_motif(seqs1, reference_freqs=None, weights=None, align_first=False, gap_reduce=None, alphabet=None)
        fn = opj('palmotif', 'tests', 'compute.svg')
        svg_logo(motif, fn)
        self.assertTrue(True)

    def test_reference_freqs_motif(self):
        motif = compute_motif(seqs1, reference_freqs=uniprot_frequency, weights=None, align_first=False, gap_reduce=None, alphabet=None)
        fn = opj('palmotif', 'tests', 'ref_freq.svg')
        svg_logo(motif, fn)
        self.assertTrue(True)

    def test_compute_motif(self):
        motif = compute_relative_motif(seqs1, seqs2)
        fn = opj('palmotif', 'tests', 'relative_motif.svg')
        svg_logo(motif, fn)
        self.assertTrue(True)
   

if __name__ == '__main__':
    unittest.main()
